package com.dzeio.openhealth.ui.browse

import android.Manifest
import android.content.SharedPreferences
import android.os.Build
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Toast
import androidx.activity.result.contract.ActivityResultContracts
import androidx.navigation.fragment.findNavController
import androidx.preference.PreferenceManager
import com.dzeio.openhealth.R
import com.dzeio.openhealth.core.BaseFragment
import com.dzeio.openhealth.databinding.FragmentBrowseBinding
import com.dzeio.openhealth.utils.PermissionsManager
import com.google.android.material.card.MaterialCardView
import dagger.hilt.android.AndroidEntryPoint

@AndroidEntryPoint
class BrowseFragment :
    BaseFragment<BrowseViewModel, FragmentBrowseBinding>(BrowseViewModel::class.java) {

    override val bindingInflater: (LayoutInflater, ViewGroup?, Boolean) -> FragmentBrowseBinding
        get() = FragmentBrowseBinding::inflate

    private val settings: SharedPreferences by lazy {
        PreferenceManager.getDefaultSharedPreferences(requireContext())
    }

    private lateinit var button: MaterialCardView
    private val activityResult = registerForActivityResult(
        ActivityResultContracts.RequestMultiplePermissions()
    ) {
        if (it.containsValue(false)) {
            // TODO: Show a popup with choice to change it
            Toast.makeText(requireContext(), R.string.permission_declined, Toast.LENGTH_LONG).show()
            return@registerForActivityResult
        }

        button.callOnClick()
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        binding.weight.setOnClickListener {
            findNavController().navigate(BrowseFragmentDirections.actionNavBrowseToNavListWeight())
        }

        binding.waterIntake.setOnClickListener {
            findNavController().navigate(BrowseFragmentDirections.actionNavBrowseToNavWaterHome())
        }

        binding.steps.setOnClickListener {
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.Q) {
                val activityPermission = PermissionsManager.hasPermission(
                    requireContext(),
                    Manifest.permission.ACTIVITY_RECOGNITION
                )

                val notificationPermission = Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU && PermissionsManager.hasPermission(
                        requireContext(),
                        Manifest.permission.POST_NOTIFICATIONS
                    )

                val permissionsToAsk = arrayListOf<String>()

                if (!activityPermission) {
                    permissionsToAsk.add(Manifest.permission.ACTIVITY_RECOGNITION)
                }

                if (!notificationPermission && Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU) {
                    permissionsToAsk.add(Manifest.permission.POST_NOTIFICATIONS)
                }

                if (permissionsToAsk.isNotEmpty()) {
                    button = binding.steps
                    activityResult.launch(permissionsToAsk.toTypedArray())
                    return@setOnClickListener
                }
            }
            findNavController().navigate(
                BrowseFragmentDirections.actionNavBrowseToStepsHomeFragment()
            )
        }

        viewModel.steps.observe(viewLifecycleOwner) {
            binding.stepsText.setText("$it of xxx steps")
        }

    }
}
