package com.dzeio.openhealth.ui.weight

import android.view.LayoutInflater
import androidx.core.content.ContextCompat
import androidx.lifecycle.lifecycleScope
import com.dzeio.openhealth.R
import com.dzeio.openhealth.core.BaseDialog
import com.dzeio.openhealth.data.weight.Weight
import com.dzeio.openhealth.databinding.DialogAddWeightBinding
import com.dzeio.openhealth.ui.home.HomeViewModel
import com.google.android.material.dialog.MaterialAlertDialogBuilder
import dagger.hilt.android.AndroidEntryPoint
import kotlinx.coroutines.flow.collect

@AndroidEntryPoint
class AddWeightDialog : BaseDialog<HomeViewModel, DialogAddWeightBinding>(HomeViewModel::class.java) {

    override val bindingInflater: (LayoutInflater) -> DialogAddWeightBinding = DialogAddWeightBinding::inflate

    override fun onBuilderInit(builder: MaterialAlertDialogBuilder) {
        super.onBuilderInit(builder)

        builder.apply {
            setTitle("Add your weight (kg)")
            setIcon(activity?.let { ContextCompat.getDrawable(it, R.drawable.ic_outline_timeline_24) })
            setPositiveButton("Validate") { _, _ ->
                save()
            }
            setNegativeButton("Cancel") { dialog, _ ->
                dialog.cancel()
            }
        }
    }

    override fun onCreated() {
        super.onCreated()

        lifecycleScope.launchWhenStarted {
            viewModel.lastWeight().collect {
                if (it != null) {
                    binding.kg.value = it.weight.toInt()
                    binding.gram.value = ((it.weight - it.weight.toInt()) * 10).toInt()
                }
            }
        }

        binding.kg.maxValue = 636
        binding.kg.minValue = 0

        binding.gram.maxValue = 9
        binding.gram.minValue = 0
    }

    private fun save() {
        val weight = Weight().apply {
            weight = binding.kg.value + (binding.gram.value.toFloat() / 10)
            source = "OpenHealth"
        }
        lifecycleScope.launchWhenCreated {
            viewModel.addWeight(weight)
        }
        // callback?.invoke()
        dialog?.dismiss()
    }
}
