package com.dzeio.openhealth.ui.weight

import android.content.SharedPreferences
import android.os.Bundle
import android.view.*
import androidx.lifecycle.lifecycleScope
import androidx.navigation.fragment.findNavController
import androidx.preference.PreferenceManager
import androidx.recyclerview.widget.LinearLayoutManager
import com.dzeio.openhealth.R
import com.dzeio.openhealth.adapters.WeightAdapter
import com.dzeio.openhealth.core.BaseFragment
import com.dzeio.openhealth.data.weight.Weight
import com.dzeio.openhealth.databinding.FragmentListWeightBinding
import com.dzeio.openhealth.graphs.WeightChart
import com.dzeio.openhealth.units.WeightUnit
import com.dzeio.openhealth.utils.GraphUtils
import com.google.android.material.color.MaterialColors
import dagger.hilt.android.AndroidEntryPoint
import kotlinx.coroutines.flow.collectLatest

@AndroidEntryPoint
class ListWeightFragment :
    BaseFragment<ListWeightViewModel, FragmentListWeightBinding>(ListWeightViewModel::class.java) {

    override val bindingInflater: (LayoutInflater, ViewGroup?, Boolean) ->
    FragmentListWeightBinding = FragmentListWeightBinding::inflate

    val settings: SharedPreferences by lazy {
        PreferenceManager.getDefaultSharedPreferences(requireContext())
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        // FIXME: deprecated
        setHasOptionsMenu(true)

        if (viewModel.goalWeight != null) {
            binding.goalButton.setText(R.string.edit_goal)
        }

        val recycler = binding.list

        val manager = LinearLayoutManager(requireContext())
        recycler.layoutManager = manager

        val adapter = WeightAdapter()

        val unit = settings.getString("weight_unit", "Kilogram") ?: "Kilogram"

        adapter.unit = WeightUnit.fromSettings(unit)

        adapter.onItemClick = {
            findNavController().navigate(
                ListWeightFragmentDirections.actionNavListWeightToNavEditWeight(
                    it.id
                )
            )
            // EditWeightDialog().show(requireActivity().supportFragmentManager, "dialog")
        }
        recycler.adapter = adapter

        viewLifecycleOwner.lifecycleScope.launchWhenCreated {
            viewModel.fetchWeights().collectLatest {
                updateGraph(it)
                val itt = it.toMutableList()
                itt.sortWith { o1, o2 -> if (o1.timestamp > o2.timestamp) -1 else 1 }
                adapter.set(itt)
            }
        }

        GraphUtils.lineChartSetup(
            binding.chart,
            MaterialColors.getColor(
                requireView(),
                com.google.android.material.R.attr.colorPrimary
            ),
            MaterialColors.getColor(
                requireView(),
                com.google.android.material.R.attr.colorOnBackground
            )
        )
    }

    private fun updateGraph(list: List<Weight>) {
        WeightChart.setup(
            binding.chart,
            requireView(),
            list,
            viewModel.weightUnit,
            viewModel.goalWeight?.toFloat(),
            false
        )
    }

    override fun onPrepareOptionsMenu(menu: Menu) {
        menu.findItem(R.id.action_add).isVisible = true

        super.onPrepareOptionsMenu(menu)
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        return when (item.itemId) {
            R.id.action_add -> {
                findNavController().navigate(
                    ListWeightFragmentDirections.actionNavListWeightToNavAddWeightDialog()
                )
                true
            }
            else -> super.onOptionsItemSelected(item)
        }
    }
}
