package com.dzeio.charts.axis

import android.graphics.Canvas
import android.graphics.Paint
import android.graphics.RectF
import com.dzeio.charts.Entry

sealed interface XAxisInterface {

    /**
     * enable/disable the display of the xAxis
     */
    var enabled: Boolean

    /**
     * set X position
     */
    var x: Double

    /**
     * X increment
     */
    var increment: Double

    /**
     * text Paint
     */
    val textPaint: Paint

    /**
     * indicate the max number of entries are displayed
     */
    var displayCount: Int

    /**
     * indicate the number of labels displayed
     */
    var labelCount: Int

    /**
     * run when manually refreshing the system
     *
     * this is where the pre-logic is handled to make [onDraw] quicker
     */
    fun refresh()

    /**
     * get the entry position on the rect
     *
     * @return the left side of the position of the entry
     */
    fun getPositionOnRect(entry: Entry, rect: RectF): Double

    /**
     * get the graph offset for X (kinda like [getPositionOnRect])
     */
    fun getXOffset(rect: RectF): Double

    /**
     * get the maximum the X can get to
     */
    fun getXMax(): Double

    /**
     * get the minimum the X can get to
     */
    fun getXMin(): Double

    /**
     * onDraw event that will draw the XAxis
     *
     * @param canvas the canvas to draw on
     * @param space the space where it is allowed to draw
     *
     * @return the final height of the XAxis
     */
    fun onDraw(canvas: Canvas, space: RectF): Float
}